#ifndef _PyGSL_FEATURES_CONFIG_H_
#define _PyGSL_FEATURES_CONFIG_H_ 1

/*
 * WARNING: Automatically generated file: do not edit by hand.
 *          edit 'setup_try' instead!
 */
/* check that no config variables are predefined */

#ifdef PYGSL_GSL_CONFIG_GSL_VERSION
#warning The name PYGSL_GSL_CONFIG_GSL_VERSION 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* PYGSL_GSL_CONFIG_GSL_VERSION */

#ifdef PYGSL_GSL_MAJOR_VERSION
#warning The name PYGSL_GSL_MAJOR_VERSION 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* PYGSL_GSL_MAJOR_VERSION */

#ifdef PYGSL_GSL_MINOR_VERSION
#warning The name PYGSL_GSL_MINOR_VERSION 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* PYGSL_GSL_MINOR_VERSION */

#ifdef _PYGSL_GSL_HAS_BSPLINE
#warning The name _PYGSL_GSL_HAS_BSPLINE 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_BSPLINE */

#ifdef _PYGSL_GSL_HAS_CONST_CGS
#warning The name _PYGSL_GSL_HAS_CONST_CGS 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_CONST_CGS */

#ifdef _PYGSL_GSL_HAS_CONST_CGSM
#warning The name _PYGSL_GSL_HAS_CONST_CGSM 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_CONST_CGSM */

#ifdef _PYGSL_GSL_HAS_CONST_MKS
#warning The name _PYGSL_GSL_HAS_CONST_MKS 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_CONST_MKS */

#ifdef _PYGSL_GSL_HAS_CONST_MKSA
#warning The name _PYGSL_GSL_HAS_CONST_MKSA 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_CONST_MKSA */

#ifdef _PYGSL_GSL_HAS_DERIV
#warning The name _PYGSL_GSL_HAS_DERIV 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_DERIV */

#ifdef _PYGSL_GSL_HAS_GSL_EIGEN_FRANCIS_WORKSPACE
#warning The name _PYGSL_GSL_HAS_GSL_EIGEN_FRANCIS_WORKSPACE 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_EIGEN_FRANCIS_WORKSPACE */

#ifdef _PYGSL_GSL_HAS_GSL_EIGEN_NONSYMM_PARAMS
#warning The name _PYGSL_GSL_HAS_GSL_EIGEN_NONSYMM_PARAMS 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_EIGEN_NONSYMM_PARAMS */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_ACCUM
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_ACCUM 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_ACCUM */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_MATRIX
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_MATRIX 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_MATRIX */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_CHOLESKY_INVERT
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_CHOLESKY_INVERT 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_CHOLESKY_INVERT */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_DECOMP
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_DECOMP 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_DECOMP */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_INVERT
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_INVERT 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_INVERT */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SOLVE
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SOLVE 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SOLVE */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SVX
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SVX 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SVX */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_HOUSEHOLDER_MH
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_HOUSEHOLDER_MH 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_HOUSEHOLDER_MH */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_DECOMP
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_DECOMP 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_DECOMP */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SET_ZERO
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SET_ZERO 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SET_ZERO */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SUBMATRIX
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SUBMATRIX 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SUBMATRIX */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK_ACCUM
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK_ACCUM 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK_ACCUM */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_HESSTRI_DECOMP
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_HESSTRI_DECOMP 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_HESSTRI_DECOMP */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_QR_MATQ
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_QR_MATQ 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_QR_MATQ */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_QR_QTMAT
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_QR_QTMAT 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_QR_QTMAT */

#ifdef _PYGSL_GSL_HAS_GSL_LINALG_SV_LEVERAGE
#warning The name _PYGSL_GSL_HAS_GSL_LINALG_SV_LEVERAGE 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_LINALG_SV_LEVERAGE */

#ifdef _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_CYCLES
#warning The name _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_CYCLES 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_CYCLES */

#ifdef _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_TO_LINEAR
#warning The name _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_TO_LINEAR 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_TO_LINEAR */

#ifdef _PYGSL_GSL_HAS_GSL_PERMUTATION_INVERSIONS
#warning The name _PYGSL_GSL_HAS_GSL_PERMUTATION_INVERSIONS 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_PERMUTATION_INVERSIONS */

#ifdef _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_CYCLES
#warning The name _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_CYCLES 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_CYCLES */

#ifdef _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_TO_CANONICAL
#warning The name _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_TO_CANONICAL 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_TO_CANONICAL */

#ifdef _PYGSL_GSL_HAS_GSL_PERMUTATION_MUL
#warning The name _PYGSL_GSL_HAS_GSL_PERMUTATION_MUL 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_GSL_PERMUTATION_MUL */

#ifdef _PYGSL_GSL_HAS_INTERP2D
#warning The name _PYGSL_GSL_HAS_INTERP2D 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_INTERP2D */

#ifdef _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_PLM_ARRAY
#warning The name _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_PLM_ARRAY 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_PLM_ARRAY */

#ifdef _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_SPHPLM_ARRAY
#warning The name _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_SPHPLM_ARRAY 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_SPHPLM_ARRAY */

#ifdef _PYGSL_GSL_HAS_MULTFIT_FDFSOLVER_LMNIEL
#warning The name _PYGSL_GSL_HAS_MULTFIT_FDFSOLVER_LMNIEL 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTFIT_FDFSOLVER_LMNIEL */

#ifdef _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_NMAX_PMAX
#warning The name _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_NMAX_PMAX 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_NMAX_PMAX */

#ifdef _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_N_P
#warning The name _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_N_P 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_N_P */

#ifdef _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_JAC
#warning The name _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_JAC 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_JAC */

#ifdef _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_STRUCT_MEMBER_J
#warning The name _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_STRUCT_MEMBER_J 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_STRUCT_MEMBER_J */

#ifdef _PYGSL_GSL_HAS_MULTFIT_ROBUST
#warning The name _PYGSL_GSL_HAS_MULTFIT_ROBUST 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTFIT_ROBUST */

#ifdef _PYGSL_GSL_HAS_MULTIMIN_FDFMINIMIZER_VECTOR_BFGS2
#warning The name _PYGSL_GSL_HAS_MULTIMIN_FDFMINIMIZER_VECTOR_BFGS2 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTIMIN_FDFMINIMIZER_VECTOR_BFGS2 */

#ifdef _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX
#warning The name _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX */

#ifdef _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2
#warning The name _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2 */

#ifdef _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2RAND
#warning The name _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2RAND 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2RAND */

#ifdef _PYGSL_GSL_HAS_MULTIMIN_FSOLVER
#warning The name _PYGSL_GSL_HAS_MULTIMIN_FSOLVER 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_MULTIMIN_FSOLVER */

#ifdef _PYGSL_GSL_HAS_ODEIV2
#warning The name _PYGSL_GSL_HAS_ODEIV2 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_ODEIV2 */

#ifdef _PYGSL_GSL_HAS_RNG_KNUTHRAN2002
#warning The name _PYGSL_GSL_HAS_RNG_KNUTHRAN2002 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_RNG_KNUTHRAN2002 */

#ifdef _PYGSL_GSL_HAS_WAVELET
#warning The name _PYGSL_GSL_HAS_WAVELET 
#warning is used as preprocessor directive which is automatically defined during 
#warning the config process. The variable given here could be overwritten by the 
#warning definitions below.
#endif /* _PYGSL_GSL_HAS_WAVELET */
/* end check that no config variables are predefined */

/* definition of config variables */
#define PYGSL_GSL_CONFIG_GSL_VERSION "2.7"
#define PYGSL_GSL_MAJOR_VERSION GSL_MAJOR_VERSION
#define PYGSL_GSL_MINOR_VERSION GSL_MINOR_VERSION
#define _PYGSL_GSL_HAS_BSPLINE 1
#define _PYGSL_GSL_HAS_CONST_CGS 1
#define _PYGSL_GSL_HAS_CONST_CGSM 1
#define _PYGSL_GSL_HAS_CONST_MKS 1
#define _PYGSL_GSL_HAS_CONST_MKSA 1
#define _PYGSL_GSL_HAS_DERIV 1
#define _PYGSL_GSL_HAS_GSL_EIGEN_FRANCIS_WORKSPACE 1
#define _PYGSL_GSL_HAS_GSL_EIGEN_NONSYMM_PARAMS 1
#define _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_ACCUM 1
#define _PYGSL_GSL_HAS_GSL_LINALG_BALANCE_MATRIX 1
#define _PYGSL_GSL_HAS_GSL_LINALG_CHOLESKY_INVERT 1
#define _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_DECOMP 1
#define _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_INVERT 1
#define _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SOLVE 1
#define _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_CHOLESKY_SVX 1
#define _PYGSL_GSL_HAS_GSL_LINALG_COMPLEX_HOUSEHOLDER_MH 1
#define _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_DECOMP 1
#define _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SET_ZERO 1
#define _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_SUBMATRIX 1
#define _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK 1
#define _PYGSL_GSL_HAS_GSL_LINALG_HESSENBERG_UNPACK_ACCUM 1
#define _PYGSL_GSL_HAS_GSL_LINALG_HESSTRI_DECOMP 1
#define _PYGSL_GSL_HAS_GSL_LINALG_QR_MATQ 1
#define _PYGSL_GSL_HAS_GSL_LINALG_QR_QTMAT 1
#define _PYGSL_GSL_HAS_GSL_LINALG_SV_LEVERAGE 1
#define _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_CYCLES 1
#define _PYGSL_GSL_HAS_GSL_PERMUTATION_CANONICAL_TO_LINEAR 1
#define _PYGSL_GSL_HAS_GSL_PERMUTATION_INVERSIONS 1
#define _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_CYCLES 1
#define _PYGSL_GSL_HAS_GSL_PERMUTATION_LINEAR_TO_CANONICAL 1
#define _PYGSL_GSL_HAS_GSL_PERMUTATION_MUL 1
#define _PYGSL_GSL_HAS_INTERP2D 1
#define _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_PLM_ARRAY 1
#define _PYGSL_GSL_HAS_LINK_GSL_SF_LEGENDRE_SPHPLM_ARRAY 1
#define _PYGSL_GSL_HAS_MULTFIT_FDFSOLVER_LMNIEL 1
#define _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_NMAX_PMAX 1
#define _PYGSL_GSL_HAS_MULTFIT_LINEAR_WORKSPACE_STRUCT_MEMBER_N_P 1
#define _PYGSL_GSL_HAS_MULTFIT_NLIN_FDFSOLVER_JAC 1
#define _PYGSL_GSL_HAS_MULTFIT_ROBUST 1
#define _PYGSL_GSL_HAS_MULTIMIN_FDFMINIMIZER_VECTOR_BFGS2 1
#define _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX 1
#define _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2 1
#define _PYGSL_GSL_HAS_MULTIMIN_FMINIMIZER_NMSIMPLEX2RAND 1
#define _PYGSL_GSL_HAS_ODEIV2 1
#define _PYGSL_GSL_HAS_RNG_KNUTHRAN2002 1
#define _PYGSL_GSL_HAS_WAVELET 1
/* end definition of config variables */


#endif /* _PyGSL_FEATURES_CONFIG_H_ */
