#ifndef _PyGSL_OLD_NUMPY_COMPAT_H_
#define _PyGSL_OLD_NUMPY_COMPAT_H_ 1

#if (NPY_API_VERSION <= 0x0000004)

#define NPY_ARRAY_C_CONTIGUOUS NPY_CONTIGUOUS
#define NPY_ARRAY_WRITEABLE    NPY_WRITEABLE
#define NPY_ARRAY_ALIGNED      NPY_ALIGNED
#define NPY_ARRAY_NOTSWAPPED   NPY_NOTSWAPPED

#endif /* (NPY_API_VERSION <= 0x0000004) */

#endif  /* _PyGSL_OLD_NUMPY_COMPAT_H_ */
