/* 
 * Author: Pierre Schnizer
 * Previously features were included based on the GSL version the code was 
 * compiled against.
 * Now python setup.py config writes pygsl_lite_features_config. There preprocessor
 * variables are defined for the different available features.
 * $Id$
 */
#ifndef _PYGSL_FEATURES_H_
#define _PYGSL_FEATURES_H_ 1

#include <pygsl_lite/pygsl_lite_features_config.h>
#endif /* _PYGSL_FEATURES_H_ */
